@@ -1,7 +1,7 @@
 # __init__.py
 # -*- coding: utf-8 -*-
 #
-# Copyright  Stephen Day
+# Copyright © Stephen Day
 #
 # This module is part of Creoleparser and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -11,11 +11,11 @@ import keyword
 
 from core import Parser, ArgParser
 from dialects import (creole11_base, creole10_base, creepy10_base,
-                    create_dialect)
+                    create_dialect, parse_args)
 
 __docformat__ = 'restructuredtext en'
 
-__version__ = '0.7.2'
+__version__ = '0.7.3'
 
 creole2html = Parser(dialect=create_dialect(creole10_base), method='html')
 """This is a pure Creole 1.0 parser created for convenience"""
@@ -23,10 +23,10 @@ creole2html = Parser(dialect=create_dialect(creole10_base), method='html')
 text2html = Parser(dialect=create_dialect(creole11_base), method='html')
 """This is a Creole 1.0 parser (+ additions) created for convenience"""
 
-parse_args = ArgParser(dialect=creepy10_base(),key_func=string.lower,
-                       illegal_keys=keyword.kwlist + ['macro_name',
-                         'arg_string', 'body', 'isblock', 'environ', 'macro'])
-"""Function for parsing macro arg_strings using a relaxed xml style"""
+#parse_args = ArgParser(dialect=creepy10_base(),key_func=string.lower,
+#                       illegal_keys=keyword.kwlist + ['macro_name',
+#                         'arg_string', 'body', 'isblock', 'environ', 'macro'])
+#"""Function for parsing macro arg_strings using a relaxed xml style"""
 
 def _test():
     import doctest
