@@ -1,7 +1,7 @@
 # core.py
 # -*- coding: utf-8 -*-
 #
-# Copyright  Stephen Day
+# Copyright © Stephen Day
 #
 # This module is part of Creoleparser and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -127,9 +127,7 @@ class Parser(object):
     def preprocess(self,text):
         """This should generally be called before fragmentize().
 
-        :parameters:
-          text
-            text to be processsed.
+        :parameter text: text to be processsed.
 
         """
         text = text.replace("\r\n", "\n")
@@ -323,6 +321,10 @@ class ImplicitList(list):
     """This class marks argument lists as implicit"""
     pass
 
+class AttrDict(dict):
+    def __getattr__(self, attr):
+        return self[attr] 
+
 def _test():
     import doctest
     doctest.testmod()
