@@ -1,7 +1,7 @@
 # elements.py
 # -*- coding: utf-8 -*-
 #
-# Copyright  Stephen Day
+# Copyright © Stephen Day
 #
 # This module is part of Creoleparser and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -11,11 +11,14 @@ import re
 import urlparse
 import urllib
 import keyword
+import warnings
+import traceback
 
 import genshi.builder as bldr
 from genshi.core import Stream, Markup
 
-from core import (escape_char, esc_neg_look, fragmentize, ImplicitList) 
+from core import (escape_char, esc_neg_look, fragmentize,
+                  ImplicitList, AttrDict) 
 
 BLOCK_ONLY_TAGS = ['h1','h2','h3','h4','h5','h6',
               'ul','ol','dl',
@@ -244,6 +247,29 @@ class SimpleElement(InlineElement):
                                                           self.child_elements,
                                                           element_store, environ))
 
+class CustomElement(InlineElement):
+
+    """Finds markup defined by provided custom regexp."""
+
+    def __init__(self, reg_exp, func):
+        super(CustomElement,self).__init__('','')
+        if isinstance(reg_exp, basestring):
+            self.regexp = re.compile(esc_neg_look + re.escape(reg_exp), re.DOTALL)
+        else:
+            self.regexp = reg_exp
+        self.func = func
+
+
+    def _build(self,mo,element_store, environ):
+        if isinstance(self.func, basestring) and not isinstance(self.func, bldr.Markup):
+            value = bldr.tag(bldr.Markup(self.func))
+        else:
+            value = self.func(mo, environ)
+            if isinstance(value, basestring) and not isinstance(value, bldr.Markup):
+                raise Exception("Custom markup functions can only return Genshi objects.")
+
+        return value
+        
 
 class LinkElement(InlineElement):
 
@@ -253,7 +279,8 @@ class LinkElement(InlineElement):
     """
     
     def __init__(self,tag, token, delimiter,
-                 interwiki_delimiter,base_urls,links_funcs,default_space_char,space_chars,
+                 interwiki_delimiter,base_urls,links_funcs,
+                 interwiki_class_funcs,default_space_char,space_chars,
                  base_url,space_char,class_func,path_func):
         super(LinkElement,self).__init__(tag,token)
         self.regexp = re.compile(self.re_string(),re.DOTALL)
@@ -261,6 +288,7 @@ class LinkElement(InlineElement):
         self.interwiki_delimiter = interwiki_delimiter
         self.base_urls = base_urls
         self.links_funcs = links_funcs
+        self.class_funcs = interwiki_class_funcs
         self.default_space_char = default_space_char
         self.space_chars = space_chars
         self.base_url = base_url
@@ -268,17 +296,10 @@ class LinkElement(InlineElement):
         self.class_func = class_func
         self.path_func = path_func
         self.content_regexp = re.compile(self.content_re_string(),re.DOTALL)
-        self.arg_regexp = re.compile(self.arg_re_string(),re.DOTALL)
         self.interwikilink_regexp = re.compile(self.interwikilink_re_string())
         self.urllink_regexp = re.compile(self.urllink_re_string(), re.DOTALL)
         self.wikilink_regexp = re.compile(self.wikilink_re_string())
 
-    def arg_re_string(self):
-        key = r'((?P<key>\w+)\s*\=)?'
-        value = r'(?P<value>.*?)'
-        return r'\s*' + key + r'\s*' + value + r'\s*(?P<delimiter>' + \
-               re.escape(self.delimiter) + r'|$)(?P<tail>.*)'
-
     def content_re_string(self):
         return r'(?P<body>.*?)(' + re.escape(self.delimiter) + '(?P<arg_string>.*?))?$'
 
@@ -289,58 +310,17 @@ class LinkElement(InlineElement):
         page_name = r'(?P<page_name>\S+?( \S+?)*)' #allows any number of single spaces
         return '^' + optional_spaces + wiki_id + \
                re.escape(self.interwiki_delimiter) + ' *' + page_name + \
-               optional_spaces + '$'#+ alias
+               optional_spaces + '$'
 
     def urllink_re_string(self):
         protocol = r'^\s*((\w+?:|/)' 
         rest_of_url = r'[\S\n]*?)\s*$'
-        return protocol + rest_of_url #+ alias
+        return protocol + rest_of_url
 
     def wikilink_re_string(self):
         optional_spaces = ' *'
         page_name = r'(?P<page_name>\S+?( \S+?)*?)' #allows any number of single spaces
-        return '^' + optional_spaces + page_name + optional_spaces + '$'#+ \
-
-##    def parse_args(self, arg_string):
-##        args = []
-##        delimiter = True
-##        while delimiter:
-##            mo = self.arg_regexp.match(arg_string)
-##            key, value, delimiter, tail = mo.group('key'),mo.group('value'),mo.group('delimiter'), mo.group('tail')
-##            if key:
-##                args.append((key, value))
-##            else:
-##                args.append(value)
-##            arg_string = tail
-##        positional_args = []
-##        kw_args = {}
-##        for arg in args:
-##           if isinstance(arg,tuple):
-##             k, v  = arg
-##             k = str(k).lower()
-##             if k in keyword.kwlist:
-##                 k = k + '_'
-##             if k in kw_args:
-##                if isinstance(v,list):
-##                   try:
-##                      kw_args[k].extend(v)
-##                   except AttributeError:
-##                      v.insert(0,kw_args[k])
-##                      kw_args[k] = v
-##                elif isinstance(kw_args[k],list):
-##                   kw_args[k].append(v)
-##                else:
-##                   kw_args[k] = [kw_args[k], v]
-##                kw_args[k] = ImplicitList(kw_args[k])
-##             else:
-##                kw_args[k] = v
-##             if isinstance(kw_args[k],ImplicitList):
-##                 kw_args[k] = ','.join(kw_args[k])
-##           else:
-##             positional_args.append(arg)
-##
-##        return (positional_args, kw_args)
-
+        return '^' + optional_spaces + page_name + optional_spaces + '$'
 
     def page_name(self,mo):
         if 'wiki_id' in mo.groupdict():
@@ -359,6 +339,7 @@ class LinkElement(InlineElement):
             link_type = 'interwiki'
             base_url = self.base_urls.get(interwikilink_mo.group('wiki_id'))
             link_func = self.links_funcs.get(interwikilink_mo.group('wiki_id'))
+            class_func = self.class_funcs.get(interwikilink_mo.group('wiki_id'))
             page_name = self.page_name(interwikilink_mo)
             if link_func:
                 url = link_func(page_name)
@@ -366,9 +347,11 @@ class LinkElement(InlineElement):
                 url = urllib.quote(page_name.encode('utf-8'))
             if base_url:
                 url = urlparse.urljoin(base_url, url)
+            if class_func:
+                the_class = class_func(page_name)
         elif self.urllink_regexp.match(body):
             urllink_mo = self.urllink_regexp.match(body)
-            link_type = 'url'
+            link_type = 'external'
             if sanitizer.is_safe_uri(urllink_mo.group(1)):
                 url = urllink_mo.group(1)
             else:
@@ -391,7 +374,7 @@ class LinkElement(InlineElement):
             return mo.group(0)
         else:
             if arg_string is not None:
-                args, kw_args = [arg_string.strip()], {} #self.parse_args(arg_string)
+                args, kw_args = [arg_string.strip()], {} 
             else:
                 args, kw_args = [], {}
             try:
@@ -405,13 +388,13 @@ class AnchorElement(LinkElement):
 
     """Finds and builds internal, external, and interwiki links.
 
-    >>> link = AnchorElement('a',('[[',']]'),'|',
+    >>> link = AnchorElement('a',('[[',']]'),delimiter='|',
     ... interwiki_delimiter=':', 
     ... base_urls=dict(somewiki='http://somewiki.org/',
     ...                bigwiki='http://bigwiki.net/'),
-    ... links_funcs={},default_space_char='-',
+    ... links_funcs={},interwiki_class_funcs={},default_space_char='-',
     ... space_chars={'bigwiki':' '},base_url='http://somewiki.org/',
-    ... space_char='_',class_func=None,path_func=None)
+    ... space_char='_',class_func=None,path_func=None,external_links_class=None)
     
     >>> mo = link.regexp.search("[[http://www.google.com| here]]")
     >>> link._build(mo,{},None).generate().render()
@@ -430,16 +413,19 @@ class AnchorElement(LinkElement):
     '<a href="http://somewiki.org/Home_Page">Home</a>'
     
     """
-    
 
-    def __init__(self, *args, **kw_args):
-        super(AnchorElement,self).__init__(*args, **kw_args)    
+    def __init__(self, tag, token, external_links_class, *args, **kw_args):
+        super(AnchorElement,self).__init__(tag, token, *args, **kw_args)
+        self.external_links_class = external_links_class
+
 
     def emit(self,element_store, environ,link_type,body,url,the_class, alias=None):
         if alias:
             alias = fragmentize(alias,self.child_elements,element_store, environ)
         else:
             alias = body.strip()
+        if link_type == 'external':
+            the_class = self.external_links_class
         return bldr.tag.__getattr__(self.tag)(alias,
                                               href=url,
                                               class_=the_class)
@@ -447,15 +433,39 @@ class AnchorElement(LinkElement):
         
 class ImageElement(LinkElement):
 
-    def __init__(self, *args, **kw_args):
-        super(ImageElement,self).__init__(*args, **kw_args)    
+    """Processes image elements.
+
+    >>> img = ImageElement('img',('{{','}}'), delimiter='|',
+    ... interwiki_delimiter=':', 
+    ... base_urls=dict(somewiki='http://somewiki.org/',
+    ...                bigwiki='http://bigwiki.net/'),
+    ... links_funcs={},interwiki_class_funcs={},default_space_char='-',
+    ... space_chars={'bigwiki':' '},base_url='/images/',
+    ... space_char='_',class_func=None,path_func=None,disable_external=False)
+
+    >>> mo = img.regexp.search('{{ picture.jpg | An image of a house }}')
+    >>> img._build(mo,{},None).generate().render()
+    '<img src="/images/picture.jpg" alt="An image of a house" title="An image of a house"/>'
+
+    """
+
+
+    def __init__(self, tag, token, disable_external, *args, **kw_args):
+        super(ImageElement,self).__init__(tag, token, *args, **kw_args)
+        self.disable_external = disable_external
 
     def emit(self,element_store, environ,link_type,body,url,the_class, alt=None):
+
+        if self.disable_external and link_type == 'external':
+            return bldr.tag.span('External images are disabled', class_='external_image')
+
         if alt is None:
-            if link_type == 'url':
+            if link_type == 'external':
                 alt = urlparse.urlsplit(url).path.split('/')[-1]
             else:
                 alt = body.strip()
+
+            
         return bldr.tag.__getattr__(self.tag)(src=url ,alt=alt, title=alt,
                                               #class_=the_class
                                               )           
@@ -463,11 +473,15 @@ class ImageElement(LinkElement):
 
 class Link(InlineElement):
 
-    """Finds and builds links."""
+    """DEPRECIATED    """
     
     def __init__(self,tag, token):
         super(Link,self).__init__(tag,token)
         self.regexp = re.compile(self.re_string(),re.DOTALL)
+        warnings.warn("""
+Use of elements.Link is depreciated. 
+"""
+                  )
 
     def _build(self,mo,element_store, environ):
         
@@ -489,9 +503,11 @@ class Link(InlineElement):
 class Macro(WikiElement):
     r"""Finds and processes inline macro elements."""
 
-    def __init__(self, tag, token, func):
+    def __init__(self, tag, token, func, macros, arg_parser):
         super(Macro,self).__init__(tag,token , [])
         self.func = func
+        self.macros = macros
+        self.arg_parser = arg_parser
         self.regexp = re.compile(self.re_string())
 
 
@@ -522,13 +538,65 @@ class Macro(WikiElement):
         return esc_neg_look + re.escape(self.token[0]) + r'(' + MACRO_NAME + \
                content + ')' + esc_neg_look + re.escape(self.token[1])
 
+    def _macro_func(self,macro_name,arg_string,body,isblock,environ):
+
+        func = self.macros[macro_name]
+        arg_parser = func.func_dict.get('arg_parser') or self.arg_parser
+        if arg_parser:
+            pos, kw = arg_parser(arg_string)
+        else:
+            pos, kw = [], {}
+
+        def parsed_body(context='auto'):
+            if context == 'auto':
+                if isblock:
+                    child_elements = self.dialect.block_elements
+                else:
+                    child_elements = self.dialect.inline_elements
+            elif context == 'inline':
+                child_elements = self.dialect.inline_elements
+            elif context == 'block':
+                child_elements = self.dialect.block_elements
+            else:
+                child_elements = context
+            return bldr.tag(fragmentize(body, child_elements, {}, environ))
+
+                    
+        macro = AttrDict(name=macro_name,arg_string=arg_string,
+                         body=body, isblock=isblock, parsed_body=parsed_body)
+        
+        try:
+            value = func(macro,environ,*pos,**kw)
+        except TypeError , detail:
+            tag = isblock and 'pre' or 'code'
+            e = str(detail)
+            msg = re.sub(r"^\w*\(\) ", '', e)
+            mo = re.match(r'(.+)(\d+)(.+)(\d+) given\)$',msg)
+            if mo: #re.search(r'given\)$',msg):
+                #mo = re.match(r'(.+)(\d+)(.+)(\d+)(.+)$',msg)
+                msg = mo.group(1) + str(int(mo.group(2))-2) \
+                      + re.sub('arguments?','argument(s)',mo.group(3)) \
+                      + str(int(mo.group(4))-2) + ' given)'
+            value = bldr.tag.__getattr__(tag)("Macro error: '%s' %s"% (macro_name, msg),class_='macro_error')
+        except:
+            value = bldr.tag.pre('Unexpected error during macro execution!\n'
+                                 + traceback.format_exc(20) ,
+                                 class_='macro_error')
+
+        return value    
+
+
     trailing_slash = re.compile(r'(?<=[ "\'\]])/$')
     def _build(self,mo,element_store, environ):
         arg_string = re.sub(self.trailing_slash,'',mo.group(4))
-        if self.func:
+
+        if mo.group('name') in self.macros:
+            value = self._macro_func(mo.group('name'),arg_string,None,False,environ)    
+        elif self.func:
             value = self.func(mo.group('name'),arg_string,None,False,environ)
         else:
             value = None
+
         if value is None:
             return bldr.tag.code(self.token[0],bldr.tag.span(mo.group('name'),class_="macro_name"),
                            bldr.tag.span(arg_string,class_="macro_arg_string"),
@@ -536,7 +604,7 @@ class Macro(WikiElement):
         elif isinstance(value, (basestring,bldr.Fragment,bldr.Element, Stream)):
             return value
         else:
-            raise "macros can only return strings and genshi objects" 
+            raise Exception("macros can only return strings and genshi objects") 
         
 
 class BodiedMacro(Macro):
@@ -545,9 +613,8 @@ class BodiedMacro(Macro):
     Does not span across top level block markup
     (see BodiedBlockMacro's for that)."""
 
-    def __init__(self, tag, token, func):
-        super(BodiedMacro,self).__init__(tag,token , func)
-        self.func = func
+    def __init__(self, tag, token, func, macros, arg_parser):
+        super(BodiedMacro,self).__init__(tag,token , func, macros, arg_parser)
         self.regexp = re.compile(self.re_string(),re.DOTALL)
 
     def re_string(self):
@@ -578,37 +645,36 @@ class BodiedMacro(Macro):
             body = mo.group('body')
             tail = ''
                 
-                
-        
-        if self.func:
+        if mo.group('name') in self.macros:
+            value = self._macro_func(mo.group('name'),mo.group('arg_string'),body,False,environ)    
+        elif self.func:
             value = self.func(mo.group('name'),mo.group('arg_string'),body,False,environ)
         else:
             value = None
+
         if value is None:
             content_out = [self.token[0],bldr.tag.span(mo.group('name'),class_="macro_name"),
                            bldr.tag.span(mo.group('arg_string'),class_="macro_arg_string"),
                            self.token[1],bldr.tag.span(mo.group('body'),class_="macro_body"),
                            self.token[0] + '/' + mo.group('name') + self.token[1]]
-            return [bldr.tag.code(content_out,class_="unknown_macro", style="white-space:pre-wrap"),tail]
-            
+            output = bldr.tag.code(content_out,class_="unknown_macro", style="white-space:pre-wrap")
         elif isinstance(value, (basestring,bldr.Fragment, Stream)):
-            return [value,tail]
+            output = value
         else:
-            raise "macros can only return strings and genshi objects"
+            raise Exception("macros can only return strings and genshi objects")
+
+        return [output, tail]
 
        
 
-class BodiedBlockMacro(WikiElement):
+class BodiedBlockMacro(Macro):
     """Finds and processes block macros with bodies.
 
     The opening and closing tokens must be are each on a line alone without
     leading spaces. These macros can enclose other block level markup
     including pre blocks and other BodiedBlockMacro's."""
-
-
-    def __init__(self, tag, token, func):
-        super(BodiedBlockMacro,self).__init__(tag,token , func)
-        self.func = func
+    def __init__(self, tag, token, func, macros, arg_parser):
+        super(BodiedBlockMacro,self).__init__(tag,token , func, macros, arg_parser)
         self.regexp = re.compile(self.re_string(),re.DOTALL+re.MULTILINE)
 
     def re_string(self):
@@ -636,7 +702,6 @@ class BodiedBlockMacro(WikiElement):
         else:
             tail = ''
         if isinstance(processed, basestring) and not isinstance(processed,Markup):
-            #print '_process', repr(processed)
             text = ''.join([text[:mo.start()],processed,tail,
                         text[mo.end():]])
             frags = fragmentize(text,wiki_elements,element_store, environ)
@@ -678,24 +743,30 @@ class BodiedBlockMacro(WikiElement):
             body = mo.group('body')
             tail = ''
 
-        if self.func:
+
+        if mo.group('name') in self.macros:
+            value = self._macro_func(mo.group('name'),mo.group('arg_string'),body,True,environ)    
+        elif self.func:
             value = self.func(mo.group('name'),mo.group('arg_string'),body,True,environ)
         else:
             value = None
+
         if value is None:
-            return [bldr.tag.pre(self.token[0],bldr.tag.span(mo.group('name'),class_="macro_name"),
+            output = bldr.tag.pre(self.token[0],bldr.tag.span(mo.group('name'),class_="macro_name"),
                            bldr.tag.span(mo.group('arg_string'),class_="macro_arg_string"),
                            self.token[1],'\n',bldr.tag.span(mo.group('body'),class_="macro_body"),
                            self.token[0] + '/' + mo.group('name') + self.token[1],
-                           class_="unknown_macro"), tail]
+                           class_="unknown_macro")
         elif (isinstance(value, (Stream, basestring)) or
              (isinstance(value,bldr.Element) and value.tag in BLOCK_TAGS)):
-            return [value, tail]
+            output = value
         # Add a p tag if the value is a Fragment or Element that needs one
         elif isinstance(value, bldr.Fragment):
-            return [bldr.tag.p(value), tail]
+            output = bldr.tag.p(value)
         else:
-            raise "macros can only return strings and genshi objects"
+            raise Exception("macros can only return strings and genshi objects")
+
+        return [output, tail]
         
     
 class RawLink(InlineElement):
@@ -745,23 +816,16 @@ class RawLink(InlineElement):
 
 class URLLink(WikiElement):
     
-    """Used to find url type links inside a link.
-
-    The scope of these is within link markup only (i.e., [[url]]
-
-    >>> url_link = URLLink('a','|')
-    >>> mo = url_link.regexp.search(" http://www.google.com| here ")
-    >>> url_link.href(mo)
-    'http://www.google.com'
-    >>> url_link._build(mo,{},None).generate().render()
-    '<a href="http://www.google.com">here</a>'
-    
-    """
+    """DEPRECIATED    """
 
     def __init__(self, tag,delimiter):
         super(URLLink,self).__init__(tag, '')
         self.delimiter = delimiter
         self.regexp = re.compile(self.re_string(),re.DOTALL)
+        warnings.warn("""
+Use of elements.URLLink is depreciated. 
+"""
+                  )
 
     def re_string(self):
         protocol = r'^\s*((\w+?:|/)' 
@@ -794,23 +858,7 @@ class URLLink(WikiElement):
 
 class InterWikiLink(WikiElement):
 
-    """Used to match interwiki links inside a link.
-
-    The search scope for these is only inside links. 
-
-    >>> interwiki_link = InterWikiLink('a',
-    ... delimiter1=':', delimiter2 = '|',
-    ... base_urls=dict(somewiki='http://somewiki.org/',
-    ...                bigwiki='http://bigwiki.net/'),
-    ...                links_funcs={},default_space_char='_',
-    ...                space_chars={})
-    >>> mo = interwiki_link.regexp.search(" somewiki:Home Page|steve ")
-    >>> interwiki_link.href(mo)
-    'http://somewiki.org/Home_Page'
-    >>> interwiki_link.alias(mo,{},None)
-    ['steve']
-    
-    """
+    """DEPRECIATED    """
 
     def __init__(self, tag, delimiter1,
                  delimiter2,base_urls,links_funcs,default_space_char,space_chars):
@@ -823,6 +871,10 @@ class InterWikiLink(WikiElement):
         self.default_space_char = default_space_char
         self.space_chars = space_chars
         self.regexp = re.compile(self.re_string())
+        warnings.warn("""
+Use of elements.InterWikiLink is depreciated. 
+"""
+                  )
 
     def re_string(self):
         #all_wikis = set(self.links_funcs.keys() + self.base_urls.keys())
@@ -872,19 +924,7 @@ class InterWikiLink(WikiElement):
 
 class WikiLink(WikiElement):
 
-    """Used to match wiki links inside a link.
-
-    The search scope for these is only inside links.
-
-    >>> wiki_link = WikiLink('a','|',base_url='http://somewiki.org/',
-    ...                      space_char='_',class_func=None, path_func=None)
-    >>> mo = wiki_link.regexp.search(" Home Page |Home")
-    >>> wiki_link.href(mo)
-    'http://somewiki.org/Home_Page'
-    >>> wiki_link.alias(mo,{},None)
-    ['Home']
-    
-    """
+    """DEPRECIATED    """
 
     def __init__(self, tag, delimiter,
                  base_url,space_char,class_func,path_func):
@@ -895,6 +935,11 @@ class WikiLink(WikiElement):
         self.class_func = class_func
         self.path_func = path_func
         self.regexp = re.compile(self.re_string())
+        warnings.warn("""
+Use of elements.WikiLink is depreciated. 
+"""
+                  )
+
 
     def re_string(self):
         optional_spaces = ' *'
@@ -1231,47 +1276,19 @@ class TableCell(WikiElement):
         return esc_neg_look + re.escape(self.token) + whitespace + \
                content + whitespace + look_ahead    
 
-
-
-##class Link(InlineElement):
-##
-##    """Finds and builds links."""
-##    
-##    def __init__(self,tag, token):
-##        super(Link,self).__init__(tag,token)
-##
-##    def _build(self,mo,element_store, environ):
-##        
-##        for tag in self.child_elements:
-##            m = tag.regexp.search(mo.group(1))
-##            if m:
-##                link = tag._build(m,element_store, environ)
-##                if link:
-##                    break
-##        else:
-##            link = None
-##
-##        if link:
-##            return bldr.tag(link)
-##        else:
-##            return mo.group(0)
-
 class Image(InlineElement):
 
-    """Processes image elements.
-
-    >>> img = Image('img',('{{','}}'), delimiter='|')
-    >>> mo = img.regexp.search('{{ picture.jpg | An image of a house }}')
-    >>> img._build(mo,{},None).generate().render()
-    '<img src="picture.jpg" alt="An image of a house" title="An image of a house"/>'
-
-    """
+    """DEPRECIATED    """
 
     def __init__(self, tag, token, delimiter):
         super(Image,self).__init__(tag,token )
         self.regexp = re.compile(self.re_string())
         self.delimiter = delimiter
         self.src_regexp = re.compile(r'^\s*(\S+)\s*$')
+        warnings.warn("""
+Use of elements.Image is depreciated. 
+"""
+                  )
 
     def _build(self,mo,element_store, environ):
         body = mo.group(1).split(self.delimiter,1)
@@ -1453,7 +1470,7 @@ class LineBreak(InlineElement):
 
     def re_string(self):
         if self.blog_style:
-            return '(' + esc_neg_look + re.escape(self.token) + r'|\n(?!$))'
+            return '(' + esc_neg_look + re.escape(self.token) + r'\n?|\n(?!$))'
         else:
             return esc_neg_look + re.escape(self.token)
     
